<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM - Open Source CRM application.
 * Copyright (C) 2014-2021 Yurii Kuznietsov, Taras Machyshyn, Oleksii Avramenko
 * Website: https://www.espocrm.com
 *
 * EspoCRM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EspoCRM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EspoCRM. If not, see http://www.gnu.org/licenses/.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

namespace Espo\Core\Api;

use Espo\Core\Exceptions\Error;

use Espo\Core\Authentication\{
    Authentication,
};

/**
 * Builds Auth instance.
 */
class AuthBuilder
{
    private $authRequired = false;

    private $isEntryPoint = false;

    private $authentication = null;

    public function setAuthentication(Authentication $authentication) : self
    {
        $this->authentication = $authentication;

        return $this;
    }

    public function setAuthRequired(bool $authRequired) : self
    {
        $this->authRequired = $authRequired;

        return $this;
    }

    public function forEntryPoint() : self
    {
        $this->isEntryPoint = true;

        return $this;
    }

    public function build() : Auth
    {
        if (!$this->authentication) {
            throw new Error("Authentication is not set.");
        }

        if ($this->isEntryPoint) {
            return Auth::createForEntryPoint($this->authentication, $this->authRequired);
        }

        return new Auth($this->authentication, $this->authRequired);
    }
}
